package com.shixing.sxedit;

import com.shixing.sxedit.types.SXFileType;
import com.shixing.sxedit.types.SXResourceType;
import com.shixing.sxedit.util.Size;
/**
 * 资源
 * */
public class SXResource {

    /**
     * 通过类型和资源路径创建资源
     *
     * <p>
     *     1. 使用完成后, 调用 destroy 方法释放
     * </p>
     * @param type {@link SXResourceType}
     * @param sourcePath 资源路径
     * */
    public SXResource(SXResourceType type, String sourcePath) {
        mNativeResource = nCreate(sourcePath, type.value);
        mIsCreateByUser = true;
    }

    /**
     * 通过合成创建资源
     *
     * <p>
     *     1. 使用完成后, 调用 destroy 方法释放
     * </p>
     * @param composite, {@link SXComposite}
     * */
    public SXResource(SXComposite composite) {
        mNativeResource = nCreateWithComposite(composite.getNativeEditManager(), composite.getCompositeId());
        mIsCreateByUser = true;
    }

    public SXResource(long nativeSource) {
        mNativeResource = nativeSource;
    }
    /**
     * 销毁创建的资源对象
     * */
    public void destroy() {
        if (mNativeResource > 0 && mIsCreateByUser) {
            nDelete(mNativeResource);
            mNativeResource = 0;
        }
    }

    /**
     * 是否有效
     *
     * @return boolean
     * */
    public boolean isValid() {
        if (mNativeResource > 0) {
            return nIsValid(mNativeResource);
        }
        return false;
    }

    /**
     * 获取资源路径
     *
     * @return 资源本地路径
     * */
    public String getResourcePath() {
        if (mNativeResource > 0) {
            return nResourcePath(mNativeResource);
        }
        return null;
    }

    /**
     * 获取 packageId
     *
     * <p>
     *     仅 Package 文件类型资源有效
     * </p>
     *
     * @return packageId
     * */
    public String getPackageId() {
        if (mNativeResource > 0) {
            return nPackageId(mNativeResource);
        }
        return null;
    }

    /**
     * 获取描述信息
     * <p>
     *     Package 文件类型的资源一般会带有描述信息
     * </p>
     *
     * @return String, 描述
     * */
    public String getDescription() {
        if (mNativeResource > 0) {
            return nDescription(mNativeResource);
        }
        return null;
    }

    /**
     * 获取资源类型
     *
     * @return {@link SXResourceType}
     * */
    public SXResourceType getType() {
        if (mNativeResource > 0) {
            return SXResourceType.values()[nResourceType(mNativeResource)];
        }
        return null;
    }

    /**
     * 获取资源尺寸大小
     *
     * @return 有返回 Size, 没有返回 null
     * */
    public Size getSize() {
        if (mNativeResource > 0) {
            int[] size = nSize(mNativeResource);
            if (size != null) {
                return new Size(size[0], size[1]);
            }
        }
        return null;
    }

    /**
     * 获取时长
     * @return 单位为秒
     */
    public double getDuration() {
        if (mNativeResource > 0) {
            return nDuration(mNativeResource);
        }
        return 0;
    }
    /**
     * 获取文件类型
     * @return 文件类型
     */
    public SXFileType getFileType() {
        if (mNativeResource > 0) {
            return SXFileType.values()[nFileType(mNativeResource)];
        }
        return SXFileType.UnknownFile;
    }
    /**
     * 获取音频采样率
     * @return 采样率，单位Hz
     */
    public double getSampleRate() {
        if (mNativeResource > 0){
            return nSampleRate(mNativeResource);
        }
        return 0;
    }
    /**
     * 获取音频声道数
     * @return 声道数
     */
    public int getChannels() {
        if (mNativeResource > 0) {
            return nChannels(mNativeResource);
        }
        return 0;
    }
    /**
     * 获取纹理id，仅纹理类型有效
     * @return 纹理id
     */
    public long getTextureId() {
        if (mNativeResource > 0) {
            return nTextureId(mNativeResource);
        }
        return 0;
    }

    /**
     * 获取合成 Id
     *
     * @return String, 合成ID
     * */
    public String getCompositeId() {
        if (mNativeResource > 0){
            return nCompositeId(mNativeResource);
        }
        return null;
    }

    @Override
    protected void finalize() {
        destroy();
    }

    private long mNativeResource;
    private boolean mIsCreateByUser = true;


    public long getNativeResource() {
        return mNativeResource;
    }

    static native long nCreate(String sourcePath, int type);
    static native long nCreateWithComposite(long nativeManager, String compositeId);
    static native void nDelete(long nativeSource);
    static native boolean nIsValid(long nativeSource);
    static native String nResourcePath(long nativeSource);
    static native String nPackageId(long nativeSource);
    static native String nDescription(long nativeSource);
    static native int nResourceType(long nativeSource);
    static native int[] nSize(long nativeSource);
    static native double nDuration(long nativeSource);
    static native int nFileType(long nativeSource);
    static native double nSampleRate(long nativeSource);
    static native int nChannels(long nativeSource);
    static native long nTextureId(long nativeSource);
    static native String nCompositeId(long nativeSource);
}
