package com.shixing.sxedit;

import com.shixing.sxedit.util.Size;

import java.io.Serializable;

/**
 * 轨道基类
 *
 * */

public abstract class SXTrack extends SXBaseImpl implements Serializable{

    public static final int SX_TRACK_TYPE_MEDIA = 0;
    public static final int SX_TRACK_TYPE_STICKER = 1;
    public static final int SX_TRACK_TYPE_TEXT = 2;
    public static final int SX_TRACK_TYPE_AUDIO = 3;
    /**
     * 轨道是否可用
     *
     * @return true表示可用，false表示不可用
     */
    public boolean isValid() {
        if (mNativeManager > 0) {
            return nIsValid(mNativeManager, mOwnType.value,mTrackID);
        }
        return false;
    }

    /**
     * 获取轨道类型
     *
     * @return 轨道类型
     */
    public int getType() {
        return mTrackType;
    }

    /**
     * 获取轨道ID
     *
     * @return 轨道ID
     */
    public String getTrackId() {
        return mTrackID;
    }

    /**
     * 获取组ID
     *
     * @return 组ID
     */
    public String getGroupId() {
        if (mNativeManager > 0) {
            return nGroupId(mNativeManager, mOwnType.value,mTrackID);
        }
        return null;
    }

    /**
     * 设置是否启用
     *
     * @param enable 是否启用
     */
    public void setEnabled(boolean enable) {
        if (mNativeManager > 0) {
            nSetEnabled(mNativeManager, mOwnType.value, mTrackID, enable);
        }
    }
    /**
     * 获取是否启用
     * @return true表示启用
     */
    public boolean isEnabled() {
        if (mNativeManager > 0) {
            return nEnabled(mNativeManager, mOwnType.value,mTrackID);
        }
        return false;
    }

    /**
     * 设置是否循环
     *
     * @param loop 是否循环
     * */
    public void setLoop(boolean loop) {
        if (mNativeManager > 0) {
            nSetLoop(mNativeManager, mOwnType.value, mTrackID, loop);
        }
    }

    /**
     * 获取轨道是否循环
     *
     * @return boolean
     * */
    public boolean isLoop() {
        if (mNativeManager > 0) {
            return nLoop(mNativeManager, mOwnType.value,mTrackID);
        }
        return false;
    }

    /**
     * 获取轨道的尺寸
     *
     * @return int[]
     */
    public Size getTrackSize() {
        if (mNativeManager > 0) {
            int[] size = nTrackSize(mNativeManager,mOwnType.value, mTrackID);
            if (size != null) {
                return new Size(size[0], size[1]);
            }
        }
        return null;
    }

    /**
     * 获取轨道开始显示的时间
     *
     * @return double
     * */
    public double getDisplayTime() {
        if (mNativeManager > 0) {
            return nDisplayTime(mNativeManager,mOwnType.value, mTrackID);
        }
        return 0;
    }

    /**
     * 获取轨道的起始时间
     *
     * @return 起始时间
     */
    public double getStartTime() {
        if (mNativeManager > 0) {
            return nStartTime(mNativeManager,mOwnType.value, mTrackID);
        }
        return 0;
    }
    /**
     * 获取轨道自身的偏移时间
     *
     * <p>
     *     1. 偏移时间（offset time）= 显示时间（display time）- 轨道开始时间（start time）
     * </p>
     *
     * @return double, 单位为秒
     */
    public double getOffsetTime() {
        if (mNativeManager > 0) {
            return nOffsetTime(mNativeManager,mOwnType.value, mTrackID);
        }
        return 0;
    }

    /**
     * 获取轨道的时长
     *
     * @return 轨道时长
     */
    public double getDuration() {
        if (mNativeManager > 0) {
            return nDuration(mNativeManager,mOwnType.value, mTrackID);
        }
        return 0;
    }

    /**
     * 整体移动轨道到指定时间点
     *
     * <p>
     *    1. 以轨道开始显示时间(display time)为基准
     * </p>
     *
     * @param displayTime 轨道开始显示的时间
     * */
    public void moveToTime(double displayTime) {
        if (mNativeManager > 0) {
            nMoveToTime(mNativeManager, mOwnType.value,mTrackID, displayTime);
        }
    }

    /**
     * 设置轨道的开始时间，即整体移动轨道到对应的时间
     *
     * <p>
     *  1. 以轨道开始时间(start time)为基准
     * </p>
     * @param startTime 轨道的开始时间
     */
    public void setStartTime(double startTime) {
        if (mNativeManager > 0) {
            nSetStartTime(mNativeManager, mOwnType.value,mTrackID, startTime);
        }
    }

    /**
     * 设置轨道的自身的偏移时间
     *
     * <p>
     *  轨道开始时间（start time） + 偏移时间（offset time） = 显示时间（display time）
     * </p>
     *
     * @param time 偏移时长，须在轨道素材时长范围内
     */
    public void setOffsetTime(double time) {
        if (mNativeManager > 0) {
            nSetOffsetTime(mNativeManager, mOwnType.value,mTrackID, time);
        }
    }
    /**
     * 设置轨道时长
     *
     * @param duration, 时长, 单位 : 秒
     * */
    public void setDuration(double duration) {
        if (mNativeManager > 0) {
            nSetDuration(mNativeManager, mOwnType.value,mTrackID, duration);
        }
    }
    /**
     * 判断是否在指定时间点,轨道是否可见
     *
     * @param time 时间
     * @return boolean
     */
    public boolean isVisibleAtTime(double time) {
        if (mNativeManager > 0) {
            return nVisibleAtTime(mNativeManager, mOwnType.value,mTrackID, time);
        }
        return false;
    }

    @Override
    public String toString() {
        String type = null;
        switch (getType()) {
            case SX_TRACK_TYPE_MEDIA:
                type = "Media";
                break;
            case SX_TRACK_TYPE_STICKER:
                type = "Sticker";
                break;
            case SX_TRACK_TYPE_TEXT:
                type = "Text";
            case SX_TRACK_TYPE_AUDIO:
                type = "Audio";
                break;
        }
        return "SXTrack[type=" + type + ", id=" + mTrackID + "]";
    }

    @Override
    public boolean equals(Object obj) {
        return mTrackID != null && obj instanceof SXTrack && mTrackID.equals(((SXTrack) obj).mTrackID);
    }


    /**
     * track 所属类型
     * */
    protected enum SXTrackOwnType {
        UnKnown(0),
        Editor(1),
        Composite(2);

        public final int value;
        SXTrackOwnType(int _value) {
            value = _value;
        }
    }

    protected long mNativeManager;
    protected String mTrackID;
    protected int mTrackType;
    protected SXTrackOwnType mOwnType;

    SXTrack(long track, long manager, SXTrackOwnType type) {
        mTrackID = nTrackId(track);
        mNativeManager = manager;
        mOwnType = type;
    }

    static SXTrack createNewTrack(long track, long manager, SXTrack.SXTrackOwnType ownType) {
        int type = nType(track);
        switch (type) {
            case SX_TRACK_TYPE_MEDIA:
                return new SXMediaTrack(track, manager, ownType);
            case SX_TRACK_TYPE_STICKER:
                return new SXStickerTrack(track, manager, ownType);
            case SX_TRACK_TYPE_TEXT:
                return new SXTextTrack(track, manager, ownType);
            case SX_TRACK_TYPE_AUDIO:
                return new SXAudioTrack(track, manager, ownType);
            default:
                return null;
        }
    }
    @Override
    long getNativeHandle() {
        return nGetNativeTrack(mNativeManager, mOwnType.value, mTrackID);
    }

    /** SXVETrack **/
    static native long nGetNativeTrack(long nativeManager, int type, String trackID);
    static native String nTrackId(long nativeTrack);
    static native int nType(long nativeTrack);
    static native boolean nIsValid(long nativeManager, int type, String trackID);
    static native boolean nEnabled(long nativeManager, int type,String trackID);
    static native void nSetEnabled(long nativeManager, int type, String trackID, boolean enable);
    static native void nSetLoop(long nativeManager, int type,String trackID, boolean loop);
    static native boolean nLoop(long nativeManager, int type,String trackID);

    static native String nGroupId(long nativeManager, int type,String trackID);
    static native void nMoveToTime(long nativeManager, int type,String trackID, double time);
    static native void nSetStartTime(long nativeManager, int type,String trackID, double time);
    static native void nSetOffsetTime(long nativeManager, int type,String trackID, double time);
    static native void nSetDuration(long nativeManager, int type,String trackID, double duration);

    static native double nStartTime(long nativeManager, int type,String trackID);
    static native double nDuration(long nativeManager, int type,String trackID);
    static native double nOffsetTime(long nativeManager,int type, String trackID);
    static native double nDisplayTime(long nativeManager, int type,String trackID);
    static native boolean nVisibleAtTime(long nativeManager, int type,String trackID, double time);


    /** SXVERenderTrack **/
    static native float[] nTransform(long nativeManager, int type,String trackID);
    static native void nSetTransform(long nativeManager, int type, String trackID, float[] transform);
    static native float[] nPosition(long nativeManager, int type, String trackID);
    static native void nSetPosition(long nativeManager, int type, String trackID, float[] position);
    static native void nSetScale(long nativeManager, int type, String trackID, float scaleX, float scaleY);
    static native float[] nScale(long nativeManager, int type, String trackID);
    static native void nSetRotation(long nativeManager,int type,  String trackID, float degrees);
    static native float nRotation(long nativeManager, int type, String trackID);
    static native void nSetOpacity(long nativeManager,int type,  String trackID, float opacity);
    static native float nOpacity(long nativeManager, int type, String trackID);

    static native void nSetVerticalFlip(long nativeManager,int type, String trackID, boolean verticalFlip);
    static native boolean nVerticalFlip(long nativeManager,int type, String trackID);
    static native void nSetHorizontalFlip(long nativeManager,int type, String trackID, boolean horizontalFlip);
    static native boolean nHorizontalFlip(long nativeManager,int type, String trackID);
    static native void nFitToComposite(long nativeManager,int type, String trackID, boolean coverAll, int margin);
    static native int[] nTrackSize(long nativeManager,int type, String trackID);


    static native long nAddFilter(long nativeManager, int type, String trackID, String filterPath, double duration, int index);
    static native long nAddFilterByCopy(long nativeManager, int type, String trackID, long nativeFilter, int index);
    static native boolean nRemoveFilter(long nativeManager, int type,String trackID, String filterId);
    static native long nGetFilter(long nativeManager, int type, String trackId, String filterId);
    native static long[] nGetFilters(long nativeManager,int type, String trackID);
    static native int nNumFilters(long nativeManager,int type, String trackId);

    static native long nAddAnimation(long nativeManager, int type,String trackID, String animationPath, double duration, int index);
    static native long nAddAnimationByCopy(long nativeManager,int type, String trackID, long nativeAnimation, int index);
    static native boolean nRemoveAnimation(long nativeManager, int type,String trackID, String animationId);
    static native long nGetAnimation(long nativeManager, int type,String trackId, String effectId);
    native static long[] nGetAnimations(long nativeManager, int type,String trackID);
    static native int nNumAnimations(long nativeManager, int type,String trackId);

    static native long nAddVideoEffect(long nativeManager,int type, String trackID, String videoEffectPath, double duration, int index);
    static native long nAddVideoEffectByCopy(long nativeManager, int type, String trackID, long nativeVideoEffect, int index);
    static native boolean nRemoveVideoEffect(long nativeManager, int type, String trackID, String videoEffectId);
    static native long nGetVideoEffect(long nativeManager, int type, String trackId, String effectId);
    native static long[] nGetVideoEffects(long nativeManager, int type, String trackID);
    static native int nNumVideoEffects(long nativeManager, int type, String trackId);

    static native long nAddGenericEffect(long nativeManager,int type, String trackId, long genericEffectType, double duration, int index);
    static native long nAddGenericEffectByCopy(long nativeManager, int type, String trackID, long nativeVideoEffect, int index);
    static native boolean nRemoveGenericEffect(long nativeManager,int type, String trackId, String effectId);
    static native long nGetGenericEffect(long nativeManager, int type,String trackId, String effectId);
    native static long[] nGetGenericEffects(long nativeManager, int type,String trackID);
    static native int nNumGenericEffects(long nativeManager, int type, String trackId);

    static native long nAddTrackAnimation(long nativeManager, int type, String trackID, String animationPath, double duration, int index);
    static native long nAddTrackAnimationByCopy(long nativeManager, int type, String trackID, long nativeAnimation, int index);
    static native boolean nRemoveTrackAnimation(long nativeManager, int type, String trackID, String animationId);
    static native long nGetTrackAnimation(long nativeManager, int type, String trackId, String effectId);
    native static long[] nGetTrackAnimations(long nativeManager, int type, String trackID);
    static native int nNumTrackAnimations(long nativeManager, int type, String trackId);

    /** SXVETextTrack **/
    static native String nContent(long nativeManager, int type, String trackID);
    static native void nSetContent(long nativeManager, int type, String trackID, String content);
    static native float nStrokeWidth(long nativeManager,int type, String trackID);
    static native void nSetStrokeWidth(long nativeManager,int type, String trackID, float strokeWidth);
    static native float nFontSize(long nativeManager,int type, String trackID);
    static native void nSetFontSize(long nativeManager,int type, String trackID, float fontSize);
    static native int nTracking(long nativeManager,int type, String trackID);
    static native void nSetTracking(long nativeManager,int type, String trackID, int tracking);
    static native String nFontFile(long nativeManager,int type, String trackID);
    static native void nSetFontFile(long nativeManager,int type, String trackID, String fontFile);
    static native int nStrokeStyle(long nativeManager,int type, String trackID);
    static native void nSetStrokeStyle(long nativeManager,int type, String trackID, int style);
    static native int nDirection(long nativeManager,int type, String trackID);
    static native void nSetDirection(long nativeManager,int type, String trackID, int direction);
    static native int nAlignment(long nativeManager, int type,String trackID);
    static native void nSetAlignment(long nativeManager,int type, String trackID, int ordinal);
    static native void nSetFillColor(long nativeManager, int type,String trackID, float red, float green, float blue);
    static native void nSetFillColorByHexString(long nativeManager, int type,String trackID, String hexString);
    static native void nSetStrokeColor(long nativeManager, int type,String trackID, float red, float green, float blue);
    static native void nSetStrokeColorByHexString(long nativeManager, int type,String trackID, String hexString);

    static native boolean nSetBubble(long nativeManager, int type,String trackID, String resourcePath);
    static native void nRemoveBubble(long nativeManager, int type,String trackID);
    static native long nBubbleResource(long nativeManager,int type, String trackID);
    static native boolean nSetTextStyle(long nativeManager,int type, String trackID, String resourcePath);
    static native void nResetTextStyle(long nativeManager, int type,String trackID);
    static native long nStyleResource(long nativeManager,int type, String trackID);

    static native long nAddTextAnimation(long nativeManager,int type, String trackID, String textAnimationPath, int index);
    static native long nAddTextAnimationByCopy(long nativeManager,int type, String trackID, long nativeTextAnimation, int index);
    static native boolean nRemoveTextAnimation(long nativeManager,int type, String trackID, String textAnimationId);
    static native long nGetTextAnimation(long nativeManager,int type, String trackID, String textAnimationId);
    static native int nNumTextAnimations(long nativeManager,int type, String trackID);
    static native long[] nGetTextAnimations(long nativeManager,int type, String trackID);


}
